#!/bin/sh
#-------------------------------------------------------------------------------
lockfile=${1}
dir=${2}
WORKDIR=${3}
PESDIR=${4}
scratch=${5}
qcpath=${6}
ncore=${7}
save_chk=${8}
save_fchk=${9}
orig_chkfile=${10}
qctype=${11}
mpipath=${12}
search_chk=${13}
task=${14}
#-------------------------------------------------------------------------------
export qcdir=$(dirname $qcpath)
scriptdir=$(dirname $0)
#-------------------------------------------------------------------------------
if [ -z $dir  ]; then
	exit 1
fi
#-------------------------------------------------------------------------------
# Created scratch directory
#
if [ "${scratch}x" = "x" ]; then
  scratch=${PESDIR}/$dir.$$
else
  scratch=$scratch/$dir.$$
fi
#
mkdir -p $scratch
#-------------------------------------------------------------------------------
if [ ${qctype}x = "gaussianx" ]; then
	export GAUSS_EXEDIR=$qcdir
	export PATH="$GAUSS_EXEDIR:$PATH"
	readchk=`grep "#" ${PESDIR}/$dir.com | grep -i "guess=read" | wc -l`
	gausopt=`grep "#" ${PESDIR}/$dir.com | grep -i "opt"    | wc -l`
	ifforce=`grep "#" ${PESDIR}/$dir.com | grep -i "force"  | wc -l`
	iffreq=` grep "#" ${PESDIR}/$dir.com | grep -i "freq"   | wc -l`
	ifnosy=` grep "#" ${PESDIR}/$dir.com | grep -i "nosymm" | wc -l`
	chkfile="$dir.chk"
	checkfile="$dir.chk"
	#-------------------------------------------------------------------------------
	if [ $ifforce -eq 0 -a $task == "numfreq" ]; then
		sed 's/^\(#\).*/& force/i' ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
		mv ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
	fi
	#-------------------------------------------------------------------------------
	if [ $ifnosy -eq 0 -a $task == "numfreq" ]; then
		sed 's/^\(#\).*/& nosymm/i' ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
		mv ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
	fi
	#-------------------------------------------------------------------------------
	if [ ${search_chk}x = "yesx" ]; then
		# debug
		ls ${PESDIR}/*.chk >& /dev/null
		exist_fchk=$?
		if [ ${exist_fchk} -eq 0 ]; then
			flst="$dir.lst"
			ls -lrt $PESDIR/*.chk | awk '{print $9}' > $flst
			fmin=`$scriptdir/search_nearest_file.exe $dir $flst`
			fmin=${fmin/.chk/}
			if [ -s $PESDIR/$fmin.chk ]; then
				echo "copy chk: $fmin.chk -> $dir.chk" | tee $PESDIR/$dir.inf
				cp ${PESDIR}/$fmin.chk $scratch/$dir.chk
				if [ $readchk -eq 0 ]; then
					if [ $gausopt -eq 0 ]; then
						sed 's/^\(#\).*/& guess=read/i' ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
						mv ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
					fi
				fi
			fi
			rm -rf $flst
		fi
	fi
	#-------------------------------------------------------------------------------
	nline=`grep -i '^%chk'        ${PESDIR}/$dir.com | wc -l`
	if [ $nline -eq 0 ]; then
		sed 1"i\%chk=$checkfile"   ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
		mv   ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
	else
		sed "s/^\(%chk\).*/%chk=$chkfile/i" ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
		mv   ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
	fi
	#-------------------------------------------------------------------------------
	#if [ $readchk -eq 1 -a ${search_chk}x != "yesx" ]; then
	#	echo "cp ${WORKDIR}/${orig_chkfile} ${scratch}/$chkfile"
	#	cp ${WORKDIR}/${orig_chkfile} ${scratch}/$chkfile
	#fi
	if [ ${search_chk}x = "yesx" ]; then
		if [ $readchk -eq 1 -a ${exist_fchk} -ne 0 ]; then
			sed '/^#/s/guess=read//i' ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
			mv   ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
		fi
	fi
	#-------------------------------------------------------------------------------
	# set %nprocs=$ncore
	#
	if [ $ncore -gt 0 ]; then
		nline=`grep -i -n '^%nprocs'          ${PESDIR}/$dir.com | wc -l`
		if [ $nline -eq 0 ]; then
			sed 1"i\%nprocs=$ncore"            ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
			mv   ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
		else
			sed "s/^\(%nprocs\).*/%nprocs=$ncore/i" ${PESDIR}/$dir.com > ${PESDIR}/$dir.com.$$
			mv   ${PESDIR}/$dir.com.$$ ${PESDIR}/$dir.com
		fi
	fi
	#-------------------------------------------------------------------------------
	# Do Gaussian calculation
	#
	cd $scratch
	cp ${PESDIR}/$dir.com $scratch
	#-------------------------------------------------------------------------------
	echo "$qcpath $dir.com > $scratch/$dir.log "
	#
	$qcpath $dir.com > $scratch/$dir.log
	calc_success=`tail -n 1 $scratch/$dir.log | grep "Normal termination" | wc -l`
	if [ ${calc_success} -eq 0 ]; then
		echo "Error termination in" $scratch/$dir.log
		touch ${PESDIR}/$dir.err
	else
		touch ${PESDIR}/$dir.yes
	fi
	#-------------------------------------------------------------------------------
	# Copy Gaussian log file and delete the scratch directory
	#
	if [ ${save_fchk}x = "yesx" -a ${calc_success} -eq 1 ]; then
		$qcdir/formchk $dir.chk >& /dev/null
		n1=`ls $dir 2> /dev/null | grep "$machine" | wc -l`
		if [ $n1 -eq 1 ]; then
			mv $scratch/$dir ${PESDIR}/$dir.fchk
		fi
	fi
	if [ ${save_chk}x = "yesx" -a ${calc_success} -eq 1 ]; then
		mv $scratch/$dir.chk ${PESDIR}
	fi
	mv $scratch/$dir.com ${PESDIR}
	mv $scratch/$dir.log ${PESDIR}
elif [ ${qctype}x = "turbomolex" ]; then
	#----------------------------------------------------------------------------
	# Calculate TURBOMOLE
	#
	#
	export TURBODIR=/export/home/turbomole/TURBOMOLE
	export PATH=$TURBODIR/bin/em64t-unknown-linux-gnu:$TURBODIR/scripts:$PATH
	#
	export PARNODES=$ncore
# 	export PARA_ARCH=SMP
# 	export PARA_ARCH=MPI
	#
	echo $scratch
	cd  ${WORKDIR}
	echo "cpc $scratch"
	cpc $scratch
	cp  ${WORKDIR}/sing_a $scratch
	#
	cd $scratch
	cp ${PESDIR}/$dir.com $scratch/coord
	echo "dscf > $dir.log"
	$qcdir/dscf   > $dir.log
	$qcdir/egrad >> $dir.log
	mv $scratch/$dir.log ${PESDIR}/$dir.log
elif [ ${qctype}x = "molprox" ]; then
	#----------------------------------------------------------------------------
	# Calculate molpro
	#
	export MOLPRO_SCRDIR=$scratch
	export TMPDIR=$scratch
	cd $scratch
	cp ${PESDIR}/$dir.com $scratch
	echo "$qcpath $dir.com"
	$qcpath $dir.com
	mv $scratch/$dir.com ${PESDIR}
	mv $scratch/$dir.out ${PESDIR}/$dir.log
elif [ ${qctype}x = "vaspx" ]; then
	#----------------------------------------------------------------------------
	# VASP
	#
	if [ ${search_chk}x = "yesx" ]; then
		# debug
		ls ${PESDIR}/*.chk >& /dev/null
		exist_fchk=$?
		if [ ${exist_fchk} -eq 0 ]; then
			flst="$dir.lst"
			# debug
			ls -lrt $PESDIR/*.chk | awk '{print $9}' > $flst
			fmin=`$scriptdir/search_nearest_file $dir $flst`
			# debug
			fmin=${fmin/.chk/}
			if [ -s $PESDIR/$fmin.chk ]; then
				echo "copy chk: $fmin.chk -> $dir.chk" | tee $PESDIR/$dir.inf
				#cp ${PESDIR}/$fmin.chk $scratch/$dir.chk
				cp ${PESDIR}/$fmin.chk ${scratch}/WAVECAR
			fi
			rm -rf $flst
		fi
	fi
	cp $WORKDIR/INCAR   $scratch/INCAR
	cp $WORKDIR/POTCAR  $scratch/POTCAR
	cp $PESDIR/$dir.com $scratch/POSCAR
	#cp ${PESDIR}/${dir}.chk ${scratch}/WAVECAR
	cd $scratch
	if [ $ncore -eq 1 ]; then
		$qcpath
	elif [ $ncore -gt 1 ]; then
		# setup mpd env (Of course use some other secret word than "dfadfs")
		if [ ! -f ~/.mpd.conf ]; then
			/bin/echo "secretword=dfadfs" >> ~/.mpd.conf
			/bin/chmod 600 ~/.mpd.conf
		fi
		#export LD_LIBRARY_PATH=$MKL_PATH:$mpipath/lib64:$LD_LIBRARY_PATH
		rm -rf hosts
		icore=1
		while [ $icore -le $ncore ]; do
			echo "localhost" >> hosts
			icore=`expr $icore + 1`
		done
		N_MPD=`cat hosts|uniq|wc -l`
		echo "$mpipath/bin64/mpdboot -r ssh -n $N_MPD -f hosts"
		$mpipath/bin64/mpdboot -r ssh -n $N_MPD -f hosts
		echo "$mpipath/bin64/mpiexec -machinefile hosts -n $ncore $qcpath"
		$mpipath/bin64/mpiexec -machinefile hosts -n $ncore $qcpath > OUTLOG
		#-------------------------------------------------------------------------
		mv ${scratch}/OUTCAR  ${PESDIR}/$dir.log
		if [ ${save_chk}x = "yesx" ]; then
			mv ${scratch}/OUTLOG  ${PESDIR}/$dir.out
			mv ${scratch}/WAVECAR ${PESDIR}/$dir.chk
		fi
		#-------------------------------------------------------------------------
		# clean
		echo "$mpipath/bin64/mpdallexit"
		$mpipath/bin64/mpdallexit
	fi
elif [ ${qctype}x = "pwscfx" ]; then
	#----------------------------------------------------------------------------
	# PWSCF
	#
	#export LD_LIBRARY_PATH=$MKL_PATH:$MPI_HOME/lib64:$LD_LIBRARY_PATH
	cd $scratch
	rm -rf hosts
	icore=0
	while [ $icore -lt $ncore ]; do
		echo "localhost" >> hosts
		icore=`expr $icore + 1`
	done
	cp ${PESDIR}/$dir.com $scratch
	cp -r $WORKDIR/pseudo $scratch
	pwd
	ls
	echo "$mpipath/bin/mpirun -machinefile $scratch/hosts -np $ncore $qcpath -npool 1 < $dir.com > $dir.log"
	$mpipath/bin/mpirun -machinefile $scratch/hosts -np $ncore $qcpath -npool 1 < $dir.com > $dir.log
	cp $scratch/$dir.log ${PESDIR}
elif [ ${qctype}x = "nwchemx" ]; then
	#============================================================================
	#=======================================
	# NWCHEM
	#=======================================
	#============================================================================
	cd $scratch
	cp ${PESDIR}/$dir.com $scratch
	#----------------------------------------------------------------------------
	# setup mpd env (Of course use some other secret word than "dfadfs")
	if [ ! -f ~/.mpd.conf ]; then
		/bin/echo "secretword=dfadfs" >> ~/.mpd.conf
		/bin/chmod 600 ~/.mpd.conf
	fi
	#----------------------------------------------------------------------------
	# Intel MPI Home
	MPI_HOME=$mpipath
	export MPICH_PROCESS_GROUP=no
	rm -rf hosts
	icore=1
	while [ $icore -le $ncore ]; do
		echo "localhost" >> hosts
		icore=`expr $icore + 1`
	done
	# N_MPD=1
	# $mpipath/mpdboot -r ssh -n $N_MPD -f hosts
	#----------------------------------------------------------------------------
	if [ ${search_chk}x = "yesx" ]; then
		# debug
		#ls ${PESDIR}/*.yes >& /dev/null
		ls ${PESDIR}/*.chk >& /dev/null
		exist_fchk=$?
		if [ ${exist_fchk} -eq 0 ]; then
			flst="$dir.lst"
			# debug
			ls -lrt $PESDIR/*.chk | awk '{print $9}' > $flst
			fmin=`$scriptdir/search_nearest_file $dir $flst`
			# debug
			fmin=${fmin/.chk/}
			if [ -s $PESDIR/$fmin.chk ]; then
				echo "copy chk: $fmin.chk -> $dir.chk" | tee $scratch/$dir.inf
				cp ${PESDIR}/$fmin.chk $scratch/$dir.chk
			fi
			rm -rf $flst
		fi
	fi
	#----------------------------------------------------------------------------
	nline=`grep -i 'vectors output' $dir.com | wc -l`
	if [ $nline -eq 1 ]; then
		sed -i "s/\(vectors output\).*/vectors output \"$dir.chk\"/" $dir.com
	fi
	#----------------------------------------------------------------------------
	nline=`grep -i 'vectors input' $dir.com | wc -l`
	ls ${PESDIR}/$dir.chk >& /dev/null
	exist_chk=$?
	if [ ${exist_chk} -eq 0 -a $nline -eq 1 ]; then
		sed -i "s/\(vectors input\).*/vectors input \"$dir.chk\"/" $dir.com
	elif [ ${exist_chk} -ne 0 -a $nline -eq 1 ]; then
		sed -i "/vectors input/d" $dir.com
	fi
	#----------------------------------------------------------------------------
	# run Nwchem
	#
	echo "$mpipath/mpiexec -machinefile hosts -n $ncore $qcpath $dir.com > $dir.log 2> /dev/null"
	$mpipath/mpiexec -machinefile hosts -n $ncore $qcpath $dir.com > $dir.log 2> $dir.err
	$mpipath/mpiexec -machinefile hosts -n $ncore $qcpath $dir.com > $dir.log 2> $dir.err
	success=$?
	#----------------------------------------------------------------------------
	nline=`grep -i 'vectors input'  $dir.com | wc -l`
	mline=`grep -i 'vectors output' $dir.com | wc -l`
	if [ $nline -eq 0 -a $mline -eq 1 ]; then
		sed -i "/vectors output/i\vectors input  \"$dir.chk\"" $dir.com
	fi
	#----------------------------------------------------------------------------
	# $mpipath/mpiexec -genv I_MPI_DEBUG 3 -genv I_MPI_DEVICE ssm \
	#	-machinefile hosts -n $ncore $qcpath $dir.com > $dir.log
	# $mpipath/mpdallexit
	cp $scratch/$dir.com ${PESDIR}
	cp $scratch/$dir.log ${PESDIR}
	cp $scratch/$dir.err ${PESDIR}
	if [ $success -eq 0 ]; then
		cp $scratch/$dir.inf ${PESDIR} >& /dev/null
		cp $scratch/$dir.chk ${PESDIR} >& /dev/null
	fi
	#----------------------------------------------------------------------------
elif [ ${qctype}x = "wien2kx" ]; then
	cd ${PESDIR}
	files=`grep old $dir.def | cut -d "," -f 2 | cut -d "'" -f 2`
	fout5=`grep "coord" $dir.def | grep log | cut -d "," -f 2 | cut -d "'" -f 2`
	frho1=`grep "coord" $dir.def | grep rho | cut -d "," -f 2 | cut -d "'" -f 2`
	#----------------------------------------------------------------------------
	cmd="cp $dir.def $files $scratch"
	echo $cmd
	$cmd >& /dev/null
	#----------------------------------------------------------------------------
	cd $scratch
	lapw5 $dir.def
	cmd="cp $frho1 $fout5 ${PESDIR}"
	echo $cmd
	$cmd
	#----------------------------------------------------------------------------
fi
rm -rf $scratch
rm -rf ${PESDIR}/$lockfile
